// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// ====================================================================================
// 
//  __      __      __
//  \ \    /  \    / /   [File name   ] rgb24to32.v
//   \ \  / /\ \  / /    [Description ] RGB 24bit to 32bit
//    \ \/ /  \ \/ /     [Timestamp   ] 
//     \  /    \  /      [version     ] 1.0.0
//      \/      \/
//
// ==============0ooo===================================================0ooo==========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | Caojie     | 08/26/19     | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo==========


module rgb24to32 #
(
    parameter VIN_WIDTH = 320
)
(
	input             I_rst_n     , //reset signal, low active 
	input             I_clk       ,
	input             I_vs        ,
	input             I_hs        ,
	input             I_de        ,
	input      [23:0] I_data      ,
	output            O_vs        ,
	output            O_hs        ,
	output            O_de        ,
	output     [31:0] O_data      
);

//==============================================================
localparam VOUT_WIDTH = (VIN_WIDTH>>2)+(VIN_WIDTH>>1);
localparam WIDTH_DIFF = VOUT_WIDTH*3/4;

//==============================================================
reg  [WIDTH_DIFF:0] I_vs_dn;

reg  [WIDTH_DIFF:0] I_hs_dn;

reg         I_de_d1;
reg         I_de_d2;

reg  [23:0] I_data_d1;
reg  [23:0] I_data_d2;


reg  [1:0]  dcnt;
reg         data32_en;
reg  [31:0] data32;

//--------------------
wire        fifo_rst;
wire [11:0] Rnum;
reg         fifo_cnt_flag;
reg  [11:0] rd_cnt;

reg         fifo_rden;
reg         fifo_rden_d1;
wire [31:0] fifo_rdata;

//==============================================================
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		begin
			I_vs_dn   <= {(WIDTH_DIFF+1){1'b0}};
			I_hs_dn   <= {(WIDTH_DIFF+1){1'b0}};
		end
	else
		begin
			I_vs_dn   <= {I_vs_dn[WIDTH_DIFF-1:0],I_vs};
			I_hs_dn   <= {I_hs_dn[WIDTH_DIFF-1:0],I_hs};
		end
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		begin
			I_de_d1   <= 1'b0;
			I_de_d2   <= 1'b0;
			I_data_d1 <= 24'd0;
			I_data_d2 <= 24'd0;
		end
	else
		begin
			I_de_d1   <= I_de;
			I_de_d2   <= I_de_d1;
			I_data_d1 <= I_data;
			I_data_d2 <= I_data_d1;
		end
end

//-------------------------------------------------
//24bit * 4 = 32bit * 3
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		dcnt <= 2'd0;
	else if(I_de_d1)
		dcnt <= dcnt + 1'b1;
	else
		dcnt <= 2'd0;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		data32_en <= 1'b0;
	else if(dcnt>2'd0) //dcnt = 1,2,3
		data32_en <= 1'b1;
	else
		data32_en <= 1'b0;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		begin
			data32 <= 32'd0;
		end
	else 
		begin
			case(dcnt)
				2'd1   : data32 <= {I_data_d1[ 7: 0],I_data_d2[23: 0]};
				2'd2   : data32 <= {I_data_d1[15: 0],I_data_d2[23: 8]};
				2'd3   : data32 <= {I_data_d1[23: 0],I_data_d2[23:16]};
				default: data32 <= data32;
			endcase
		end
end

//------------------------------------------------------
assign fifo_rst = ~I_de_d1 & I_de;

fifo_32x2048 fifo_32x2048_inst
(
	.Data  (data32    ), //input [31:0] Data
    .Reset (fifo_rst  ), //input Reset
	.WrClk (I_clk     ), //input WrClk
	.RdClk (I_clk     ), //input RdClk
	.WrEn  (data32_en ), //input WrEn
	.RdEn  (fifo_rden ), //input RdEn
	.Wnum  (          ), //output [11:0] Wnum
	.Rnum  (Rnum      ), //output [11:0] Rnum
	.Q     (fifo_rdata), //output [31:0] Q
	.Empty (          ), //output Empty
	.Full  (          ) //output Full
);

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		fifo_cnt_flag <= 1'd0;
	else if(fifo_rst)
		fifo_cnt_flag <= 1'd0;
    else if(Rnum >WIDTH_DIFF)
        fifo_cnt_flag <= 1'd1;
	else
		fifo_cnt_flag <= fifo_cnt_flag;
end


always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		rd_cnt <= 12'd0;
	else if(fifo_rst)
		rd_cnt <= 12'd0;
    else if(fifo_cnt_flag)
        begin
            if(rd_cnt>VOUT_WIDTH)
                rd_cnt <= rd_cnt;
            else
                rd_cnt <= rd_cnt + 1'b1;
        end
	else
		rd_cnt <= rd_cnt;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		fifo_rden <= 1'd0;
    else if(rd_cnt >=12'd1 && rd_cnt <= VOUT_WIDTH)
        fifo_rden <= 1'd1;
	else
		fifo_rden <= 1'b0;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		fifo_rden_d1 <= 1'd0;
	else
		fifo_rden_d1 <= fifo_rden;
end


//----------------------------------------
assign O_vs   = I_vs_dn[WIDTH_DIFF]   ;
assign O_hs   = I_hs_dn[WIDTH_DIFF]   ;
assign O_de   = fifo_rden_d1 ;
assign O_data = fifo_rdata;     

endmodule

