// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// ====================================================================================
// 
//  __      __      __
//  \ \    /  \    / /   [File name   ] rgb32to24.v
//   \ \  / /\ \  / /    [Description ] RGB 32bit to 24bit
//    \ \/ /  \ \/ /     [Timestamp   ] 
//     \  /    \  /      [version     ] 1.0.0
//      \/      \/
//
// ==============0ooo===================================================0ooo==========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | Caojie     | 08/26/19     | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo==========


module rgb32to24 #
(
    parameter VOUT_WIDTH = 320 ,
    parameter VSHS_DELAY = 8
)
(
	input             I_rst_n     , //reset signal, low active 
	input             I_clk       ,
	input             I_vs        ,
	input             I_hs        ,
	input             I_de        ,
	input      [31:0] I_data      ,
	output            O_vs        ,
	output            O_hs        ,
	output            O_de        ,
	output     [23:0] O_data      
);

//==============================================================
reg  [VSHS_DELAY-1:0]  I_vs_dn;

reg  [VSHS_DELAY-1:0]  I_hs_dn;

reg         I_de_d1;
reg         I_de_d2;
reg         I_de_d3;
reg         I_de_d4;

reg  [31:0] I_data_d1;
reg  [31:0] I_data_d2;

//--------------------
wire        fifo_rst;
reg         fifo_cnt_flag;
reg  [11:0] rd_cnt;
reg  [1:0]  rd_round_cnt;
reg  [1:0]  dcnt;
reg  [1:0]  dcnt_d1;
reg  [1:0]  dcnt_d2;

reg         fifo_rden;
wire [31:0] fifo_rdata;
reg  [31:0] fifo_rdata_d1;

//---------------------
reg         data24_en;
reg         data24_en_d1;
reg         data24_en_d2;
reg  [23:0] data24;

//==============================================================
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		begin
			I_vs_dn   <= {VSHS_DELAY{1'b0}};
			I_hs_dn   <= {VSHS_DELAY{1'b0}};
		end
	else
		begin
			I_vs_dn   <= {I_vs_dn[VSHS_DELAY-2:0],I_vs};
			I_hs_dn   <= {I_hs_dn[VSHS_DELAY-2:0],I_hs};
		end
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		begin
			I_de_d1   <= 1'b0;
			I_de_d2   <= 1'b0;
            I_de_d3   <= 1'b0;
			I_de_d4   <= 1'b0;
			I_data_d1 <= 32'd0;
			I_data_d2 <= 32'd0;
		end
	else
		begin
			I_de_d1   <= I_de;
			I_de_d2   <= I_de_d1;
            I_de_d3   <= I_de_d2;
			I_de_d4   <= I_de_d3;
			I_data_d1 <= I_data;
			I_data_d2 <= I_data_d1;
		end
end

//------------------------------------------------------
assign fifo_rst = ~I_de_d1 & I_de;

fifo_32x2048 fifo_32x2048_inst
(
	.Data  (I_data_d2 ), //input [31:0] Data
    .Reset (fifo_rst  ), //input Reset
	.WrClk (I_clk     ), //input WrClk
	.RdClk (I_clk     ), //input RdClk
	.WrEn  (I_de_d2   ), //input WrEn
	.RdEn  (fifo_rden ), //input RdEn
	.Wnum  (          ), //output [11:0] Wnum
	.Rnum  (          ), //output [11:0] Rnum
	.Q     (fifo_rdata), //output [31:0] Q
	.Empty (          ), //output Empty
	.Full  (          ) //output Full
);

//---------------------------------------------------
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		fifo_cnt_flag <= 1'd0;
	else if(fifo_rst)
		fifo_cnt_flag <= 1'd0;
    else if(!I_de_d4 & I_de_d3) //I_de rising edge
        fifo_cnt_flag <= 1'd1;
	else
		fifo_cnt_flag <= fifo_cnt_flag;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		rd_cnt <= 12'd0;
	else if(fifo_rst)
		rd_cnt <= 12'd0;
    else if(fifo_cnt_flag)
        begin
            if(rd_cnt>VOUT_WIDTH)
                rd_cnt <= rd_cnt;
            else
                rd_cnt <= rd_cnt + 1'b1;
        end
	else
		rd_cnt <= rd_cnt;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		rd_round_cnt <= 2'd0;
	else if(fifo_rst)
		rd_round_cnt <= 2'd0;
    else if(fifo_cnt_flag)
        begin
            if(rd_round_cnt==2'd3)
                rd_round_cnt <= 2'd0;
            else
                rd_round_cnt <= rd_round_cnt + 1'b1;
        end
	else
		rd_round_cnt <= rd_round_cnt;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
        begin
            dcnt    <= 2'd0;
            dcnt_d1 <= 2'd0;
            dcnt_d2 <= 2'd0;
        end
	else
        begin
            dcnt    <= rd_round_cnt;
            dcnt_d1 <= dcnt        ;
            dcnt_d2 <= dcnt_d1     ;
        end
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		fifo_rden <= 1'd0;
    else if(rd_cnt >=12'd1 && rd_cnt <= VOUT_WIDTH &&
            (dcnt == 2'd0 || dcnt == 2'd1 || dcnt == 2'd2))
        fifo_rden <= 1'd1;
	else
		fifo_rden <= 1'b0;
end

//---------------------
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		data24_en <= 1'd0;
    else if(rd_cnt >=12'd1 && rd_cnt <= VOUT_WIDTH)
        data24_en <= 1'd1;
	else
		data24_en <= 1'b0;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
        begin
            data24_en_d1 <= 1'd0;
            data24_en_d2 <= 1'd0;
        end
	else
        begin
            data24_en_d1 <= data24_en;
            data24_en_d2 <= data24_en_d1;
        end
end

//------------------------
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		fifo_rdata_d1 <= 32'd0;
	else
		fifo_rdata_d1 <= fifo_rdata;
end

always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		begin
			data24 <= 24'd0;
		end
	else 
		begin
			case(dcnt_d2)
				2'd0   : data24 <=  fifo_rdata[23: 0];
				2'd1   : data24 <= {fifo_rdata[15: 0],fifo_rdata_d1[31:24]};
				2'd2   : data24 <= {fifo_rdata[ 7: 0],fifo_rdata_d1[31:16]};
                2'd3   : data24 <=                    fifo_rdata_d1[31: 8];
				default: data24 <= data24;
			endcase
		end
end

//----------------------------------------
assign O_vs   = I_vs_dn[VSHS_DELAY-1]   ;
assign O_hs   = I_hs_dn[VSHS_DELAY-1]   ;
assign O_de   = data24_en_d2 ;
assign O_data = data24_en_d2 ? data24 : 24'd0;     

endmodule

