// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =					 All rights reserved.
// ====================================================================================
// 
//  __	    __	    __
//  \ \	   /  \	   / /   [File name   ] scaler_lite_test_top.v
//   \ \  / /\ \  / /	 [Description ] Video demo
//	  \ \/ /  \ \/ /	 [Timestamp   ] Friday Jan 8 14:00:30 2021
//	   \  /	   \  /	     [version	  ] 1.0
//	    \/	    \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:	|  Author	| Mod. Date	| Changes Made:
// ----------------------------------------------------------------------------------
// V1.0	| Caojie	 | 01/08/21	 | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========


//According to IP parameters to choose
`define	    WR_VIDEO_WIDTH_32  //16/24/32
`define	DEF_WR_VIDEO_WIDTH 32  //16/24/32

`define	    RD_VIDEO_WIDTH_32  //16/24/32
`define	DEF_RD_VIDEO_WIDTH 32  //16/24/32

`define	USE_THREE_FRAME_BUFFER

`define	DEF_ADDR_WIDTH 21 
`define	DEF_SRAM_DATA_WIDTH 64


// `define	VESA_RGB_24BIT
`define MIPI4LANES_RGB_32BIT

// `define	ZOOM_RATIO_2_1
`define	ZOOM_RATIO_3_2
// `define	ZOOM_RATIO_4_3

module scaler_lite_test_top
(
	input             I_clk           , //27Mhz
	input             I_rst_n         ,
	output     [1:0]  O_led           ,
	output     [1:0]  O_psram_ck      ,
	output     [1:0]  O_psram_ck_n    ,
	inout      [1:0]  IO_psram_rwds   ,
	inout      [15:0] IO_psram_dq     ,
	output     [1:0]  O_psram_reset_n ,
	output     [1:0]  O_psram_cs_n    ,
	output	   		  O_tmds_clk_p    ,
	output	   		  O_tmds_clk_n    ,
	output	   [2:0]  O_tmds_data_p   ,//{r,g,b}
	output	   [2:0]  O_tmds_data_n   
);

//=========================================================
//SRAM parameters
parameter ADDR_WIDTH          = `DEF_ADDR_WIDTH        ;   //总容量=2^21*32bit = 64Mbit ??
parameter DATA_WIDTH          = `DEF_SRAM_DATA_WIDTH   ;   //与生成PSRAM IP有关，此psram 64Mbit, x32， 则用户数据位宽固定64bit
parameter WR_VIDEO_WIDTH	  = `DEF_WR_VIDEO_WIDTH    ;  
parameter RD_VIDEO_WIDTH	  = `DEF_RD_VIDEO_WIDTH    ;  

//==================================================
reg  [31:0] run_cnt;
wire        running;

//--------------------------
wire        tp0_vs_in  ;
wire        tp0_hs_in  ;
wire        tp0_de_in ;
wire [ 7:0] tp0_data_r/*synthesis syn_keep=1*/;
wire [ 7:0] tp0_data_g/*synthesis syn_keep=1*/;
wire [ 7:0] tp0_data_b/*synthesis syn_keep=1*/;

reg         vs_r;
reg  [9:0]  cnt_vs;

//-----------------
`ifdef MIPI4LANES_RGB_32BIT 
    wire        vs32;
    wire        hs32;
    wire        de32;
    wire [31:0] data32;
`endif

//-------------------------
//Scaler
wire		scl_vin_clk;  
wire		scl_vin_vs;  
wire		scl_vin_de;
`ifdef VESA_RGB_24BIT
    wire [23:0] scl_vin_data;
`endif
`ifdef MIPI4LANES_RGB_32BIT 
    wire [31:0] scl_vin_data;
`endif

wire		sclout_vs  ;  
wire		sclout_de  ;
`ifdef VESA_RGB_24BIT
    wire [23:0] sclout_data;
`endif
`ifdef MIPI4LANES_RGB_32BIT 
    wire [31:0] sclout_data;
`endif

//-------------------------
//frame buffer in
wire                      ch0_vfb_clk_in ;
wire                      ch0_vfb_vs_in  ;
wire                      ch0_vfb_de_in  ;
wire [WR_VIDEO_WIDTH-1:0] ch0_vfb_data_in;

//-------------------
//syn_code
wire                      syn_off0_re;  // ofifo read enable signal
wire                      syn_off0_vs;
wire                      syn_off0_hs;
                          
wire                      off0_syn_de  ;
wire [RD_VIDEO_WIDTH-1:0] off0_syn_data;

//-------------------------------------
//Hyperram
wire        dma_clk  ; 

wire        memory_clk;
wire        mem_pll_lock  ;

//-------------------------------------------------
//memory interface
wire                    cmd           ;
wire                    cmd_en        ;
wire [ADDR_WIDTH-1:0]   addr          ;//[ADDR_WIDTH-1:0]
wire [DATA_WIDTH-1:0]   wr_data       ;//[DATA_WIDTH-1:0]
wire [DATA_WIDTH/8-1:0] data_mask     ;
wire                    rd_data_valid ;
wire [DATA_WIDTH-1:0]   rd_data       ;//[DATA_WIDTH-1:0]
wire                    init_calib     ;

//-----------------
wire        vs24;
wire        hs24;
wire        de24;
wire [23:0] data24;

//------------------------------------------
//rgb data
wire        rgb_vs     ;
wire        rgb_hs     ;
wire        rgb_de     ;
wire [23:0] rgb_data   ;  

//------------------------------------
//HDMI TX
wire serial_clk;
wire pll_lock;

wire hdmi_rst_n;

wire pix_clk;

//===================================================
//LED test
always @(posedge I_clk or negedge I_rst_n) //I_clk
begin
	if(!I_rst_n)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd27_000_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd13_500_000) ? 1'b1 : 1'b0;

assign  O_led[0] = running;
assign  O_led[1] = ~init_calib;

//===========================================================================
//testpattern
testpattern testpattern_inst
(
	.I_pxl_clk   (I_clk            ),//pixel clock
	.I_rst_n     (hdmi_rst_n         ),//low active 
	.I_mode      ({1'b0,cnt_vs[9:8]} ),//data select
	.I_single_r  (8'd0               ),
	.I_single_g  (8'd255             ),
	.I_single_b  (8'd0               ),                  //800x600    //1024x768   //1280x720    
	.I_h_total   (12'd1650           ),//hor total time  // 12'd1056  // 12'd1344  // 12'd1650  
	.I_h_sync    (12'd40             ),//hor sync time   // 12'd128   // 12'd136   // 12'd40    
	.I_h_bporch  (12'd220            ),//hor back porch  // 12'd88    // 12'd160   // 12'd220   
	.I_h_res     (12'd720            ),//hor resolution  // 12'd800   // 12'd1024  // 12'd1280  
	.I_v_total   (12'd750            ),//ver total time  // 12'd628   // 12'd806   // 12'd750    
	.I_v_sync    (12'd5              ),//ver sync time   // 12'd4     // 12'd6     // 12'd5     
	.I_v_bporch  (12'd20             ),//ver back porch  // 12'd23    // 12'd29    // 12'd20    
	.I_v_res     (12'd480            ),//ver resolution  // 12'd600   // 12'd768   // 12'd720    
	.I_hs_pol    (1'b1               ),//HS polarity , 0:negetive ploarity，1：positive polarity
	.I_vs_pol    (1'b1               ),//VS polarity , 0:negetive ploarity，1：positive polarity
	.O_de        (tp0_de_in          ),   
	.O_hs        (tp0_hs_in          ),
	.O_vs        (tp0_vs_in          ),
	.O_data_r    (tp0_data_r         ),   
	.O_data_g    (tp0_data_g         ),
	.O_data_b    (tp0_data_b         )
);

always@(posedge I_clk)
begin
	vs_r<=tp0_vs_in;
end

always@(posedge I_clk or negedge hdmi_rst_n)
begin
	if(!hdmi_rst_n)
		cnt_vs<=10'd0;
	else if(vs_r && !tp0_vs_in) //vs24 falling edge
		cnt_vs<=cnt_vs+1'b1;
end 

//==========================================================
`ifdef MIPI4LANES_RGB_32BIT 
    //24bit to 32bit
    rgb24to32 #
    (
        .VIN_WIDTH   (720  )
    ) 
    rgb24to32_inst
    (
        .I_rst_n     (hdmi_rst_n ), //reset signal, low active 
        .I_clk       (I_clk      ),
        .I_vs        (tp0_vs_in  ),
        .I_hs        (tp0_hs_in  ),
        .I_de        (tp0_de_in  ),
        .I_data      ({tp0_data_r,tp0_data_g,tp0_data_b}),//r,g,b
        .O_vs        (vs32       ),
        .O_hs        (hs32       ),
        .O_de        (de32       ),
        .O_data      (data32     )
    );
`endif

`ifdef VESA_RGB_24BIT 
    assign scl_vin_clk   = I_clk; 
    assign scl_vin_vs    = tp0_vs_in;
    assign scl_vin_de    = tp0_de_in;
    assign scl_vin_data  = {tp0_data_r,tp0_data_g,tp0_data_b};
`endif

`ifdef MIPI4LANES_RGB_32BIT 
    assign scl_vin_clk   = I_clk; 
    assign scl_vin_vs    = vs32;
    assign scl_vin_de    = de32;
    assign scl_vin_data  = data32;
`endif

//==============================================
Scaler_Lite_Down_Top Scaler_Lite_Down_Top_inst
(
	.I_reset		(~hdmi_rst_n          ),
	.I_vin_clk		(scl_vin_clk	      ), 
	.I_vin_vs_cpl	(~scl_vin_vs	      ), //positive					
	.I_vin_de_cpl	(scl_vin_de			  ),
`ifdef VESA_RGB_24BIT    
	.I_vin_data0_cpl(scl_vin_data[ 7: 0]  ),     
	.I_vin_data1_cpl(scl_vin_data[15: 8]  ),     
	.I_vin_data2_cpl(scl_vin_data[23:16]  ), 
`endif 
`ifdef MIPI4LANES_RGB_32BIT
    .I_vin_data0_cpl(scl_vin_data[ 7: 0]  ),     
	.I_vin_data1_cpl(scl_vin_data[15: 8]  ),     
	.I_vin_data2_cpl(scl_vin_data[23:16]  ), 
    .I_vin_data3_cpl(scl_vin_data[31:24]  ),  
`endif  
`ifdef VESA_RGB_24BIT 
	.O_vout0_data   (sclout_data[ 7: 0]   ),     		  
	.O_vout1_data   (sclout_data[15: 8]   ),     
	.O_vout2_data   (sclout_data[23:16]   ),
`endif  
`ifdef MIPI4LANES_RGB_32BIT 
    .O_vout0_data   (sclout_data[ 7: 0]   ),     		  
	.O_vout1_data   (sclout_data[15: 8]   ),     
	.O_vout2_data   (sclout_data[23:16]   ),
    .O_vout3_data   (sclout_data[31:24]   ),  
`endif 
	.O_vout_vs		(sclout_vs	          ), //positive				   
	.O_vout_de		(sclout_de	          )  
);


//==============================================
`ifdef VESA_RGB_24BIT 
    assign ch0_vfb_clk_in  = I_clk;       
    assign ch0_vfb_vs_in   = ~sclout_vs;  //negative
    assign ch0_vfb_de_in   = sclout_de;  
    assign ch0_vfb_data_in = {8'd0,sclout_data}; // RGB888
`endif

`ifdef MIPI4LANES_RGB_32BIT 
    assign ch0_vfb_clk_in  = I_clk;
    assign ch0_vfb_vs_in   = ~sclout_vs;  //vs_in;
    assign ch0_vfb_de_in   = sclout_de;  //de_in;
    assign ch0_vfb_data_in = sclout_data;//data_in;
`endif   


//=====================================================
//SRAM 控制模块 
Video_Frame_Buffer_Top Video_Frame_Buffer_Top_inst
( 
	.I_rst_n		    (init_calib       ),//rst_n            ),
	.I_dma_clk		    (dma_clk          ),   //sram_clk         ),
`ifdef USE_THREE_FRAME_BUFFER 
	.I_wr_halt		    (1'd0             ), //1:halt,  0:no halt
	.I_rd_halt		    (1'd0             ), //1:halt,  0:no halt
`endif
	// video data input           
	.I_vin0_clk		    (ch0_vfb_clk_in   ),
	.I_vin0_vs_n	    (ch0_vfb_vs_in    ),
	.I_vin0_de		    (ch0_vfb_de_in    ),
	.I_vin0_data	    (ch0_vfb_data_in  ),
	.O_vin0_fifo_full   (                 ),
	// video data output          
	.I_vout0_clk	    (pix_clk          ),
	.I_vout0_vs_n	    (~syn_off0_vs     ),
	.I_vout0_de		    (syn_off0_re      ),
	.O_vout0_den	    (off0_syn_de      ),
	.O_vout0_data	    (off0_syn_data    ),
	.O_vout0_fifo_empty (                 ),
	// ddr write request
	.O_cmd              (cmd              ),
	.O_cmd_en           (cmd_en           ),
	.O_addr             (addr             ),//[ADDR_WIDTH-1:0]
	.O_wr_data          (wr_data          ),//[DATA_WIDTH-1:0]
	.O_data_mask        (data_mask        ),
	.I_rd_data_valid    (rd_data_valid    ),
	.I_rd_data          (rd_data          ),//[DATA_WIDTH-1:0]
	.I_init_calib       (init_calib       )
); 

//================================================
//PSRAM ip
gowin_rpll gowin_rpll_inst
(
    .clkout(memory_clk    ), //output clkout
    .lock  (mem_pll_lock  ), //output lock
    .clkin (I_clk         )  //input clkin
);

PSRAM_Memory_Interface_HS_Top PSRAM_Memory_Interface_HS_Top_inst
(
	.clk            (I_clk          ),
	.memory_clk     (memory_clk     ),
	.pll_lock       (mem_pll_lock   ),
	.rst_n          (I_rst_n        ),  //rst_n
	.O_psram_ck     (O_psram_ck     ),
	.O_psram_ck_n   (O_psram_ck_n   ),
	.IO_psram_rwds  (IO_psram_rwds  ),
	.IO_psram_dq    (IO_psram_dq    ),
	.O_psram_reset_n(O_psram_reset_n),
	.O_psram_cs_n   (O_psram_cs_n   ),
	.wr_data        (wr_data        ),
	.rd_data        (rd_data        ),
	.rd_data_valid  (rd_data_valid  ),
	.addr           (addr           ),
	.cmd            (cmd            ),
	.cmd_en         (cmd_en         ),
	.clk_out        (dma_clk        ),
	.data_mask      (data_mask      ),
	.init_calib     (init_calib     )
);

//==========================================================
`ifdef MIPI4LANES_RGB_32BIT 
    `ifdef	ZOOM_RATIO_2_1
        localparam VOUT_WIDTH = 360;
    `endif 
    `ifdef	ZOOM_RATIO_3_2
        localparam VOUT_WIDTH = 480;
    `endif 
    `ifdef	ZOOM_RATIO_4_3
        localparam VOUT_WIDTH = 540;
    `endif 

    //32bit to 24bit
    rgb32to24 #
    (
        .VOUT_WIDTH   (VOUT_WIDTH  ),
        .VSHS_DELAY   (8  )
    )
    rgb32to24_inst
    (
        .I_rst_n     (hdmi_rst_n      ), //reset signal, low active 
        .I_clk       (pix_clk         ),
        .I_vs        (syn_off0_vs     ),
        .I_hs        (syn_off0_hs     ),
        .I_de        (off0_syn_de     ),
        .I_data      (off0_syn_data   ),
        .O_vs        (vs24       ),
        .O_hs        (hs24       ),
        .O_de        (de24       ),
        .O_data      (data24     )
    );
`endif   

//================================================
`ifdef VESA_RGB_24BIT
    `ifdef	ZOOM_RATIO_2_1
        localparam RD_HRES = 16'd360; 
        localparam RD_VRES = 16'd240;
    `endif 
    `ifdef	ZOOM_RATIO_3_2
        localparam RD_HRES = 16'd480; 
        localparam RD_VRES = 16'd320;
    `endif 
    `ifdef	ZOOM_RATIO_4_3
        localparam RD_HRES = 16'd540; 
        localparam RD_VRES = 16'd360;
    `endif 
`endif 

`ifdef MIPI4LANES_RGB_32BIT
    `ifdef	ZOOM_RATIO_2_1
        localparam RD_HRES = 16'd270; //360*3/4=270
        localparam RD_VRES = 16'd240;
    `endif 
    `ifdef	ZOOM_RATIO_3_2
        localparam RD_HRES = 16'd360; //480*3/4=360
        localparam RD_VRES = 16'd320;
    `endif 
    `ifdef	ZOOM_RATIO_4_3
        localparam RD_HRES = 16'd405; //540*3/4=405
        localparam RD_VRES = 16'd360;
    `endif 
`endif 

wire out_de;
syn_gen syn_gen_inst
(                                   
    .I_pxl_clk   (pix_clk         ),//40MHz      //65MHz      //74.25MHz    //148.5
    .I_rst_n     (hdmi_rst_n      ),//800x600    //1024x768   //1280x720    //1920x1080    
    .I_h_total   (16'd1650        ),// 16'd1056  // 16'd1344  // 16'd1650   // 16'd2200  
    .I_h_sync    (16'd40          ),// 16'd128   // 16'd136   // 16'd40     // 16'd44   
    .I_h_bporch  (16'd220         ),// 16'd88    // 16'd160   // 16'd220    // 16'd148   
    .I_h_res     (16'd1280        ),// 16'd800   // 16'd1024  // 16'd1280   // 16'd1920  
    .I_v_total   (16'd750         ),// 16'd628   // 16'd806   // 16'd750    // 16'd1125   
    .I_v_sync    (16'd5           ),// 16'd4     // 16'd6     // 16'd5      // 16'd5      
    .I_v_bporch  (16'd20          ),// 16'd23    // 16'd29    // 16'd20     // 16'd36      
    .I_v_res     (16'd720         ),// 16'd600   // 16'd768   // 16'd720    // 16'd1080   
    .I_rd_hres   (RD_HRES         ),
    .I_rd_vres   (RD_VRES         ),
    .I_hs_pol    (1'b1            ),//HS polarity , 0:负极性，1：正极性
    .I_vs_pol    (1'b1            ),//VS polarity , 0:负极性，1：正极性
    .O_rden      (syn_off0_re     ),
    .O_de        (out_de          ),   
    .O_hs        (syn_off0_hs     ),
    .O_vs        (syn_off0_vs     )
);

`ifdef VESA_RGB_24BIT 
    localparam N = 5; //delay N clocks
                            
    reg  [N-1:0]  Pout_hs_dn   ;
    reg  [N-1:0]  Pout_vs_dn   ;
    reg  [N-1:0]  Pout_de_dn   ;

    always@(posedge pix_clk or negedge hdmi_rst_n)
    begin
        if(!hdmi_rst_n)
            begin                          
                Pout_hs_dn  <= {N{1'b1}};
                Pout_vs_dn  <= {N{1'b1}}; 
                Pout_de_dn  <= {N{1'b0}}; 
            end
        else 
            begin                          
                Pout_hs_dn  <= {Pout_hs_dn[N-2:0],syn_off0_hs};
                Pout_vs_dn  <= {Pout_vs_dn[N-2:0],syn_off0_vs}; 
                Pout_de_dn  <= {Pout_de_dn[N-2:0],out_de}; 
            end
    end
`endif

`ifdef MIPI4LANES_RGB_32BIT 
    localparam N = 13; //delay N clocks
                            
    reg  [N-1:0]  Pout_hs_dn   ;
    reg  [N-1:0]  Pout_vs_dn   ;
    reg  [N-1:0]  Pout_de_dn   ;
    
    always@(posedge pix_clk or negedge hdmi_rst_n)
    begin
        if(!hdmi_rst_n)
            begin                          
                Pout_hs_dn  <= {N{1'b1}};
                Pout_vs_dn  <= {N{1'b1}}; 
                Pout_de_dn  <= {N{1'b0}}; 
            end
        else 
            begin                          
                Pout_hs_dn  <= {Pout_hs_dn[N-2:0],syn_off0_hs};
                Pout_vs_dn  <= {Pout_vs_dn[N-2:0],syn_off0_vs}; 
                Pout_de_dn  <= {Pout_de_dn[N-2:0],out_de}; 
            end
    end
`endif

//==============================================================================
//TMDS TX(HDMI)
`ifdef VESA_RGB_24BIT 
    assign rgb_data    = off0_syn_de ? off0_syn_data[23:0] : 24'h0000ff;//{r,g,b}
    assign rgb_vs      = Pout_vs_dn[4];//syn_off0_vs;
    assign rgb_hs      = Pout_hs_dn[4];//syn_off0_hs;
    assign rgb_de 	   = Pout_de_dn[1];//off0_syn_de;
`endif

`ifdef MIPI4LANES_RGB_32BIT
    assign rgb_data    = de24 ? data24[23:0] : 24'h0000ff;//{r,g,b}
    assign rgb_vs      = Pout_vs_dn[N-1];//syn_off0_vs;
    assign rgb_hs      = Pout_hs_dn[N-1];//syn_off0_hs;
    assign rgb_de 	   = Pout_de_dn[N-4];//off0_syn_de;
`endif

TMDS_PLL u_tmds_pll
(.clkin		(I_clk     ) 	//input clk 
,.clkout	(serial_clk) 	//output clk 
,.lock		(pll_lock  ) 	//output lock
);

assign hdmi_rst_n = I_rst_n & pll_lock;

CLKDIV u_clkdiv
(.RESETN(hdmi_rst_n)
,.HCLKIN(serial_clk) //clk  x5
,.CLKOUT(pix_clk)    //clk  x1
,.CALIB (1'b1)
);
defparam u_clkdiv.DIV_MODE="5";
defparam u_clkdiv.GSREN="false";

DVI_TX_Top DVI_TX_Top_inst
(
	.I_rst_n       (hdmi_rst_n   ),  //asynchronous reset, low active
	.I_serial_clk  (serial_clk    ),
	.I_rgb_clk     (pix_clk       ),  //pixel clock
	.I_rgb_vs      (rgb_vs        ), 
	.I_rgb_hs      (rgb_hs        ),    
	.I_rgb_de      (rgb_de        ), 
	.I_rgb_r       (rgb_data[23:16]    ),  
	.I_rgb_g       (rgb_data[15: 8]    ),  
	.I_rgb_b       (rgb_data[ 7: 0]    ),  
	.O_tmds_clk_p  (O_tmds_clk_p  ),
	.O_tmds_clk_n  (O_tmds_clk_n  ),
	.O_tmds_data_p (O_tmds_data_p ),  //{r,g,b}
	.O_tmds_data_n (O_tmds_data_n )
);



endmodule