//---------------------------------------------------------------------
// File name  : tb.v
// Module name: tb
// Created by : 
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date	     | AUTHOR	 |	DESCRIPTION
// ---------------------------------------------------------------------
//  1.0	   |  5-Mar-2021 |			 |  initial 
// ---------------------------------------------------------------------

`timescale 1ns / 1ps

//According to IP parameters to choose
// `define	VESA_RGB_24BIT
`define MIPI4LANES_RGB_32BIT

// `define	ZOOM_RATIO_2_1
`define	ZOOM_RATIO_3_2
// `define	ZOOM_RATIO_4_3

module tb();

//========================================================
//parameters
parameter BMP_VIDEO_FORMAT		   = "WxH_xHz"; //video format
parameter BMP_PIXEL_CLK_FREQ	   = 75.000;//pixel clock frequency, unit: MHz
parameter BMP_PIXEL_CLK_PERIOD	   = 1000.0/BMP_PIXEL_CLK_FREQ; //unit: ns
parameter BMP_WIDTH				   = 480;
parameter BMP_HEIGHT			   = 360;
parameter BMP_OPENED_NAME		   = "../../tb/pic/img480.bmp";
parameter BMP_REPEAT			   = 1'b1;  //0:bmp increase  , 1:bmp repeat 
parameter BMP_LINK				   = 1'b0;  //0:أ1:˫
								   
parameter BMP_OUT_VIDEO_FORMAT	   = "WxH_xHz"; //video format
parameter BMP_OUT_PIXEL_CLK_FREQ   = 75.000;//pixel clock frequency, unit: MHz
parameter BMP_OUT_PIXEL_CLK_PERIOD = 1000.0/BMP_OUT_PIXEL_CLK_FREQ; //unit: ns
`ifdef ZOOM_RATIO_2_1
    parameter BMP_OUTPUTED_WIDTH	   = 240;
    parameter BMP_OUTPUTED_HEIGHT	   = 180;
`endif
`ifdef ZOOM_RATIO_3_2
    parameter BMP_OUTPUTED_WIDTH	   = 320;
    parameter BMP_OUTPUTED_HEIGHT	   = 240;
`endif
`ifdef ZOOM_RATIO_4_3
    parameter BMP_OUTPUTED_WIDTH	   = 360;
    parameter BMP_OUTPUTED_HEIGHT	   = 270;
`endif
parameter BMP_OUTPUTED_NAME		   = "../../tb/pic/out0_001.bmp";
parameter BMP_OUTPUTED_NUMBER	   = 16'd3;

//=========================================================
parameter CONFIG_VIDEO_FORMAT	 = BMP_OUT_VIDEO_FORMAT  ;
parameter CONFIG_PIXEL_CLK_FREQ  = BMP_OUT_PIXEL_CLK_FREQ;
`ifdef VESA_RGB_24BIT
    parameter CONFIG_HOR_RESOLUTION  = BMP_OUTPUTED_WIDTH;
`endif
`ifdef MIPI4LANES_RGB_32BIT 
    parameter CONFIG_HOR_RESOLUTION  = BMP_OUTPUTED_WIDTH*3/4;
`endif
`ifdef MIPI4LANES_RGB_64BIT 
    parameter CONFIG_HOR_RESOLUTION  = BMP_OUTPUTED_WIDTH*3/8;
`endif
parameter CONFIG_VER_RESOLUTION  = BMP_OUTPUTED_HEIGHT   ;


//=========================================================
//SRAM parameters
parameter ADDR_WIDTH          = 20;	//memory unit 128bit, Total capacity=2^20*8bit = 128Mbit
parameter DATA_WIDTH          = 128;  
parameter WR_VIDEO_WIDTH	  = 32;  
parameter RD_VIDEO_WIDTH	  = 32;  
parameter SRAM_LATENCY	      = 20;
//-------------------------------------------------
parameter SRAM_CLK_FREQ	     = 100.000; //clock frequency, unit: MHz
parameter SRAM_CLK_PERIOD	 = 1000.0/SRAM_CLK_FREQ; //unit: ns  

//=======================================================
reg  pixel_clock;
reg  sram_clk;

reg  rst_n;

reg  param_update_i;

//------------
//dirver
wire	   vsync; 
wire	   hsync; 
wire	   data_valid; 
wire [7:0] data0_r; 
wire [7:0] data0_g;
wire [7:0] data0_b;

//-----------------
`ifdef MIPI4LANES_RGB_32BIT 
    wire        vs32;
    wire        hs32;
    wire        de32;
    wire [31:0] data32;
`endif

//--------
//Scaler
wire		scl_vin_clk;  
wire		scl_vin_vs;  
wire		scl_vin_de;
`ifdef VESA_RGB_24BIT
    wire [23:0] scl_vin_data;
`endif
`ifdef MIPI4LANES_RGB_32BIT 
    wire [31:0] scl_vin_data;
`endif
                       
wire		sclout_vs  ;  
wire		sclout_de  ;
`ifdef VESA_RGB_24BIT
    wire [23:0] sclout_data;
`endif
`ifdef MIPI4LANES_RGB_32BIT 
    wire [31:0] sclout_data;
`endif

//-------------------------
//frame buffer in
wire                      vfb_clk_in ;
wire                      vfb_vs_in  ;
wire                      vfb_de_in  ;
wire [WR_VIDEO_WIDTH-1:0] vfb_data_in;

//-------------------
//syn_code
reg		    rd_gen ;
wire		syn_off0_re;  // ofifo read enable signal
wire		syn_off0_vs;
wire		syn_off0_hs;

wire        syn_clip0_vs;
wire        syn_clip0_hs;

//----------------  
//config
reg		     output_pixel_clk;
wire  [15:0] data_valid_xst_o;
wire  [15:0] data_valid_xed_o;
wire  [15:0] data_valid_yst_o;
wire  [15:0] data_valid_yed_o;
wire  [15:0] hor_sync_time_o ;
wire  [15:0] ver_sync_time_o ;
wire  [15:0] hor_total_time_o;
wire  [15:0] ver_total_time_o;

//psram
wire        dma_clk  ; 

wire        memory_clk;
wire        mem_pll_lock  ;

//-------------------------------------------------
//memory interface
wire					sram_ready = 1'b1;
wire					dma_wr_en;
wire					dma_rd_en;
wire  [ADDR_WIDTH-1:0]  dma_rw_addr;
wire  [DATA_WIDTH-1:0]  dma_wr_data;
wire					dma_rd_den;
wire  [DATA_WIDTH-1:0]  dma_rd_data;

wire	                  dma_de  ;
wire [RD_VIDEO_WIDTH-1:0] dma_data;
wire                      vout0fifo_empty;

//-----------------
`ifdef MIPI4LANES_RGB_32BIT 
    wire        vs24;
    wire        hs24;
    wire        de24;
    wire [23:0] data24;
`endif

//-----------------
//monitor rgb input
wire		m_clk;
wire		m_vs_rgb;  
wire		m_hs_rgb;  
wire		m_de_rgb;  
wire [7:0]  m_data0_r;
wire [7:0]  m_data0_g;
wire [7:0]  m_data0_b;
wire [7:0]  m_data1_r;
wire [7:0]  m_data1_g;
wire [7:0]  m_data1_b;

//=====================================================
GSR GSR(.GSRI(1'b1));

//==============================================  
initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end

//=====================================================
//pixel_clk
initial
  begin
	pixel_clock	     = 1'b0;
	output_pixel_clk = 1'b0;
  end

always  #(BMP_PIXEL_CLK_PERIOD/2.0) pixel_clock = ~pixel_clock;

always  #(BMP_OUT_PIXEL_CLK_PERIOD/2.0) output_pixel_clk = ~output_pixel_clk;

//----------------------------------------------------
//ddr ref clk
initial 
begin
	sram_clk = 1'b0;
end

always  #(SRAM_CLK_PERIOD/2.0) sram_clk = ~sram_clk;

//=====================================================
//rst_n
initial
  begin
	rst_n=1'b0;
	
	#2000;
	rst_n=1'b1;
end

initial
  begin
	param_update_i=1'b0;
	
	#2050;
	param_update_i=1'b1;
	
	#2250;   //200ns ߵƽ
	param_update_i=1'b0;
end

//==================================================
//video driver
driver #
(
	.BMP_VIDEO_FORMAT	(BMP_VIDEO_FORMAT   ),
	.BMP_PIXEL_CLK_FREQ (BMP_PIXEL_CLK_FREQ ),
	.BMP_WIDTH		    (BMP_WIDTH	        ),
	.BMP_HEIGHT		    (BMP_HEIGHT	        ),
	.BMP_OPENED_NAME	(BMP_OPENED_NAME    )
)
driver_inst
(
	.link_i	       (BMP_LINK   ), 
	.repeat_en     (BMP_REPEAT ),
	.video_gen_en  (rst_n      ),//rst_n
	.pixel_clock   (pixel_clock),
	.vsync	       (vsync	   ),// 
	.hsync	       (hsync	   ),// 
	.data_valid    (data_valid ),
	.data0_r       (data0_r	   ), 
	.data0_g       (data0_g	   ),
	.data0_b       (data0_b	   ), 
	.data1_r       (	   ), 
	.data1_g       (	   ),
	.data1_b       (	   )
);

//==========================================================
//24bit to 32bit
`ifdef MIPI4LANES_RGB_32BIT 
    rgb24to32 #
    (
        .VIN_WIDTH   (BMP_WIDTH  )
    ) 
    rgb24to32_inst
    (
        .I_rst_n     (rst_n      ), //reset signal, low active 
        .I_clk       (pixel_clock),
        .I_vs        (vsync      ),
        .I_hs        (hsync      ),
        .I_de        (data_valid ),
        .I_data      ({data0_b,data0_g,data0_r}),//b,g,r
        .O_vs        (vs32       ),
        .O_hs        (hs32       ),
        .O_de        (de32       ),
        .O_data      (data32     )
    );
`endif

//==============================================
`ifdef VESA_RGB_24BIT 
    assign scl_vin_clk   = pixel_clock; 
    assign scl_vin_vs    = vsync;
    assign scl_vin_de    = data_valid;
    assign scl_vin_data  = {data0_b,data0_g,data0_r};
`endif

`ifdef MIPI4LANES_RGB_32BIT 
    assign scl_vin_clk   = pixel_clock; 
    assign scl_vin_vs    = vs32;
    assign scl_vin_de    = de32;
    assign scl_vin_data  = data32;
`endif

//==========================================================================
//scaler 
Scaler_Lite_Down_Top Scaler_Lite_Down_Top_inst
(
	.I_reset		(~rst_n			      ),
	.I_vin_clk		(scl_vin_clk	      ), 
	.I_vin_vs_cpl	(~scl_vin_vs	      ), //					
	.I_vin_de_cpl	(scl_vin_de			  ), 				          
`ifdef VESA_RGB_24BIT 
    .I_vin_data0_cpl(scl_vin_data[ 7: 0]  ),     
	.I_vin_data1_cpl(scl_vin_data[15: 8]  ),     
	.I_vin_data2_cpl(scl_vin_data[23:16]  ),
`endif   
`ifdef MIPI4LANES_RGB_32BIT 
    .I_vin_data0_cpl(scl_vin_data[ 7: 0]  ),     
	.I_vin_data1_cpl(scl_vin_data[15: 8]  ),     
	.I_vin_data2_cpl(scl_vin_data[23:16]  ),
    .I_vin_data3_cpl(scl_vin_data[31:24]  ),  
`endif    
`ifdef VESA_RGB_24BIT 
    .O_vout0_data   (sclout_data[ 7: 0]   ),     		  
	.O_vout1_data   (sclout_data[15: 8]   ),     
	.O_vout2_data   (sclout_data[23:16]   ),
`endif 	
`ifdef MIPI4LANES_RGB_32BIT 
    .O_vout0_data   (sclout_data[ 7: 0]   ),     		  
	.O_vout1_data   (sclout_data[15: 8]   ),     
	.O_vout2_data   (sclout_data[23:16]   ), 
    .O_vout3_data   (sclout_data[31:24]   ),  
`endif     
	.O_vout_vs		(sclout_vs	          ), //					   
	.O_vout_de		(sclout_de	          )  
);

//======================================================
`ifdef VESA_RGB_24BIT 
    assign vfb_clk_in  = scl_vin_clk;
    assign vfb_vs_in   = ~sclout_vs;  //vs_in;
    assign vfb_de_in   = sclout_de;  //de_in;
    assign vfb_data_in = {8'd0,sclout_data};//data_in;
`endif    

`ifdef MIPI4LANES_RGB_32BIT 
    assign vfb_clk_in  = scl_vin_clk;
    assign vfb_vs_in   = ~sclout_vs;  //vs_in;
    assign vfb_de_in   = sclout_de;  //de_in;
    assign vfb_data_in = sclout_data;//data_in;
`endif   

//==================================================================
//SRAM ģ 
vfb_sram_top vfb_sram_top_inst
( 
  .I_rst_n		     (rst_n	         ),
  .I_dma_clk		 (sram_clk	     ),
  .I_wr_halt		 (1'b0		     ),
  .I_rd_halt		 (1'b0		     ),
  // video data input 
  .I_vin0_clk		 (vfb_clk_in     ),
  .I_vin0_vs_n	     (vfb_vs_in      ),
  .I_vin0_de		 (vfb_de_in      ),
  .I_vin0_data	     (vfb_data_in    ),
  .O_vin0_fifo_full  (               ),
  // video data output		        
  .I_vout0_clk	     (output_pixel_clk   ),//output_pixel_clk),
  .I_vout0_vs_n	     (syn_off0_vs    ),//
  .I_vout0_de		 (syn_off0_re    ),
  .O_vout0_den	     (dma_de         ),
  .O_vout0_data	     (dma_data	     ),
  .O_vout0_fifo_empty(vout0fifo_empty),
  // ddr write request
  .O_wr_en	         (dma_wr_en        ),
  .O_rd_en	         (dma_rd_en        ),
  .O_wr_addr	     (dma_rw_addr      ),
  .O_wr_data	     (dma_wr_data      ),
  .I_rd_data_valid   (dma_rd_den       ),
  .I_rd_data         (dma_rd_data      ),
  .I_sram_ready      (sram_ready       )
); 

//-----------------------------------------------------  
sram_model#
(
	.ADDR_WIDTH	(ADDR_WIDTH       ),	
	.DATA_WIDTH	(DATA_WIDTH	      ),
	.LATENCY	(SRAM_LATENCY	  ) //  1~31
)
sram_model_inst
( 
	.sram_clk		(sram_clk		),
	.sram_wr_en_i	(dma_wr_en		),
	.sram_rd_en_i	(dma_rd_en		),
	.sram_rw_addr_i (dma_rw_addr    ),
	.sram_wr_data_i (dma_wr_data	),
	.sram_rd_den_o  (dma_rd_den	    ),
	.sram_rd_data_o (dma_rd_data	)
);

//=====================================================================
//ӳһʱʼ
  initial 
	begin
	  rd_gen = 1'b0 ;
	  forever
		begin
		  #100; //ns
		  if ( rst_n == 1'b1 )
			begin
			  #5000000;//ns
			  rd_gen=1'b1;
			end 
		  else 
			begin
			 rd_gen=1'b0;
			end 
		end 
	end 

//-----------------------------------------------------
//config
config_m#
(
	.OUTPUT_VIDEO_FORMAT   (CONFIG_VIDEO_FORMAT  ),
	.OUTPUT_PIXEL_CLK_FREQ (CONFIG_PIXEL_CLK_FREQ),
	.OUTPUT_HOR_RESOLUTION (CONFIG_HOR_RESOLUTION),
	.OUTPUT_VER_RESOLUTION (CONFIG_VER_RESOLUTION)
)
config_m_inst
(
	.data_valid_xst_o (data_valid_xst_o), 
	.data_valid_xed_o (data_valid_xed_o), 
	.data_valid_yst_o (data_valid_yst_o), 
	.data_valid_yed_o (data_valid_yed_o),
	.hor_sync_time_o  (hor_sync_time_o ),
	.ver_sync_time_o  (ver_sync_time_o ),
	.hor_total_time_o (hor_total_time_o), 
	.ver_total_time_o (ver_total_time_o)  
);

//---------------------------------------------------
//ͬʱģ
syn_code_16 syn_code_16_inst
(
	.cpu2out_xst_reg   (data_valid_xst_o),
	.cpu2out_xed_reg   (data_valid_xed_o),
	.cpu2out_yst_reg   (data_valid_yst_o),
	.cpu2out_yed_reg   (data_valid_yed_o),
	.cpu2out_hsync_reg (hor_sync_time_o ),
	.cpu2out_vsync_reg (ver_sync_time_o ),
	.cpu2out_hed_reg   (hor_total_time_o),
	.cpu2out_ved_reg   (ver_total_time_o),
	.hor_res0		   (data_valid_xed_o-data_valid_xst_o+1'b1),
	.ver_res0		   (data_valid_yed_o-data_valid_yst_o+1'b1),
	.vs_sel			   (1'b0			),//sim only
	.rst_n			   (rd_gen		    ),
	.pxl_clk		   (output_pixel_clk),
	.e_vs			   (1'b0			),//sim only
	.off0_syn_data	   (				),
	.syn_off0_re	   (syn_off0_re	    ),
	.syn_off0_vs	   (syn_off0_vs	    ),//
	.syn_off0_hs	   (syn_off0_hs	    ),//
	.syn_clip0_vs	   (syn_clip0_vs	),
	.syn_clip0_hs	   (syn_clip0_hs	),
	.syn_clip0_de	   (				),
	.syn_clip0_data	   (				)
);

//==========================================================
//32bit to 24bit
`ifdef MIPI4LANES_RGB_32BIT 
    rgb32to24 #
    (
        .VOUT_WIDTH   (BMP_OUTPUTED_WIDTH  ),
        .VSHS_DELAY   (8  )
    )
    rgb32to24_inst
    (
        .I_rst_n     (rst_n           ), //reset signal, low active 
        .I_clk       (output_pixel_clk),
        .I_vs        (syn_clip0_vs    ),
        .I_hs        (syn_clip0_hs    ),
        .I_de        (dma_de          ),
        .I_data      (dma_data        ),
        .O_vs        (vs24       ),
        .O_hs        (hs24       ),
        .O_de        (de24       ),
        .O_data      (data24     )
    );
`endif

//==========================================================
//monitor
`ifdef VESA_RGB_24BIT 
    assign m_clk	 = output_pixel_clk ;
    assign m_vs_rgb  = syn_clip0_vs           ;
    assign m_hs_rgb  = syn_clip0_hs           ;
    assign m_de_rgb  = dma_de           ;
    assign m_data0_r = dma_data[ 7: 0]  ;
    assign m_data0_g = dma_data[15: 8]  ;
    assign m_data0_b = dma_data[23:16]  ;
    assign m_data1_r = 8'd0	            ;
    assign m_data1_g = 8'd0	            ;
    assign m_data1_b = 8'd0	            ; 
`endif    

`ifdef MIPI4LANES_RGB_32BIT 
	assign m_clk	 = output_pixel_clk ;
	assign m_vs_rgb  = vs24           ;
	assign m_hs_rgb  = hs24           ;
	assign m_de_rgb  = de24           ;
	assign m_data0_r = data24[ 7: 0]  ;
	assign m_data0_g = data24[15: 8]  ;
	assign m_data0_b = data24[23:16]  ;
	assign m_data1_r = 8'd0	            ;
	assign m_data1_g = 8'd0	            ;
	assign m_data1_b = 8'd0	            ;
`endif  

initial 
  begin
	force m_vs_rgb = 1'b0;
	
	@(posedge monitor_inst.video2bmp_en)
	  #100000;
	  release m_vs_rgb;
  end 

monitor#
(
  .BMP_OUTPUTED_WIDTH  (BMP_OUTPUTED_WIDTH ),
  .BMP_OUTPUTED_HEIGHT (BMP_OUTPUTED_HEIGHT),
  .BMP_OUTPUTED_NAME   (BMP_OUTPUTED_NAME  ),
  .BMP_OUTPUTED_NUMBER (BMP_OUTPUTED_NUMBER)
)
monitor_inst
(
  .link_i	    (BMP_LINK	), //0,أ1˫
  .video2bmp_en (rst_n		),
  .pixel_clock  (m_clk		), 
  .vsync		(m_vs_rgb	), //	   
  .hsync		(m_hs_rgb	), //	   
  .data_valid   (m_de_rgb	), 
  .data0_r	    (m_data0_r	),	
  .data0_g	    (m_data0_g	),
  .data0_b	    (m_data0_b	),
  .data1_r	    (m_data1_r	),	
  .data1_g	    (m_data1_g	),
  .data1_b	    (m_data1_b	)
);
		  
endmodule
