// ---------------------------------------------------------------------
// File name			: sram_model.v
// Module name			: sram_model
// Module Description	: 
// Created by			:	
// ---------------------------------------------------------------------
// Release history
// VERSION 	 |	 Date	   | AUTHOR		|	DESCRIPTION
// --------------------------------------------------------------------
//	 1.0	 |	2-Jun-2014 |			|	 initial
// --------------------------------------------------------------------

`timescale 1ns / 1ps
		
module sram_model#
(
	parameter ADDR_WIDTH	= 16 ,		
	parameter DATA_WIDTH	= 256,
	parameter LATENCY		= 12	//	1~31
)
( 
	input	wire					sram_clk      ,
	input	wire					sram_wr_en_i  ,
	input	wire					sram_rd_en_i  ,
	input	wire[ADDR_WIDTH-1:0]	sram_rw_addr_i,
	input	wire[DATA_WIDTH-1:0]	sram_wr_data_i,
	output	wire					sram_rd_den_o ,
	output	wire[DATA_WIDTH-1:0]	sram_rd_data_o
);

/*************************************************************************/	
localparam		MEM_SIZE	= 2**ADDR_WIDTH;
localparam[4:0]	DEN_LATENCY	= LATENCY[4:0] ;
localparam[4:0]	D_LATENCY	= LATENCY[4:0] ;

/*************************************************************************/	
reg[DATA_WIDTH-1:0]					sram_rd_data_pi;
reg[DATA_WIDTH-1:0]					sram_mem[MEM_SIZE-1:0];

/*************************************************************************/	
reg [DEN_LATENCY-1:0]							rd_en_d	;
reg [(D_LATENCY*DATA_WIDTH)-1:0]	 rd_data_d;

/*************************************************************************/	
always@(posedge sram_clk) if (sram_wr_en_i == 1'b1) sram_mem[sram_rw_addr_i] = sram_wr_data_i;
always@(posedge sram_clk) if (sram_rd_en_i == 1'b1) sram_rd_data_pi = sram_mem[sram_rw_addr_i];

//=====================================================================
//ͨģʱ	
always@(posedge sram_clk)
begin 
	rd_en_d[DEN_LATENCY-1:0]			  <= {rd_en_d[DEN_LATENCY-2:0],sram_rd_en_i}; 
	rd_data_d[(D_LATENCY*DATA_WIDTH)-1:0] <= {rd_data_d[((D_LATENCY-1)*DATA_WIDTH)-1:0],sram_rd_data_pi}; 
end

assign	sram_rd_den_o = rd_en_d[DEN_LATENCY-1];
assign	sram_rd_data_o = rd_data_d[(D_LATENCY*DATA_WIDTH)-1:((D_LATENCY-1)*DATA_WIDTH)];
	
endmodule	 
	